import { ChangeDetectorRef, Component, EventEmitter, Input, OnInit } from '@angular/core';
import { ColumnModel } from '@syncfusion/ej2-grids';
import { ToastrService } from 'ngx-toastr';
import { Bu5Model, MyBu5Model } from 'src/app/shared/model/bu5.model';
import { Bu6Model, MyBu6Model } from 'src/app/shared/model/bu6.model';
import { Bu5Service } from 'src/app/shared/services/bu5.service';
import { Bu6Service } from 'src/app/shared/services/bu6.service';
import { FileService } from 'src/app/shared/services/file.service';

interface table {
  currentPage: number,
  page: number[],
  search: string,
  pageSize: number
}
@Component({
  selector: 'app-sub-department-three',
  templateUrl: './sub-department-three.component.html',
  styleUrls: ['./sub-department-three.component.scss']
})
export class SubDepartmentThreeComponent implements OnInit {
  bu6: { loading: boolean, select: Bu6Model, dataList: Bu6Model[] } = { loading: false, select: new MyBu6Model(), dataList: [] }

  selectedFile: File | null = null;
  selectedFileName: string = 'กรุณาเลือกไฟล์';


  bu5List: Bu5Model[] = []
  bu5: Bu5Model = new MyBu5Model()
  bu5Modal: table = {
    currentPage: 1,
    page: Array.from({ length: 1 }, (_, i) => i + 1),
    search: "",
    pageSize: 10
  }
  currentModal: 'add' | 'edit' | 'detele' = "add"

  columns: ColumnModel[] = [{
    field: "bu6id",
    headerText: "รหัสส่วนย่อย 3",
    type: "string",
    isPrimaryKey: true,
  },
  {
    field: "tdesc",
    headerText: "รายละเอียดย่อย3 (ไทย)",
    type: "string"
  },
  {
    field: "edesc",
    headerText: "รายละเอียดย่อย3 (อังกฤษ)",
    type: "string"
  }]
  searchSettings = {
    fields: ['bu6id', 'tdesc', 'edesc'],
    operator: 'contains',
    ignoreCase: false
  };
  search = ''
  selectedItems: { key: string, count: number, data: Map<string, boolean> } = { key: '', count: 0, data: new Map<string, boolean>() };
  constructor(private bu6Service: Bu6Service,
    private bu5Service: Bu5Service,
    private toastr: ToastrService,
    private cdr: ChangeDetectorRef,
    private fileService: FileService
  ) { }
  ngOnInit(): void {
    this.getBu6List()
    this.getBu5List()
  }

  onFileSelected(event: any) {
    this.selectedFile = event.target.files.length > 0 ? event.target.files[0] : null;
    this.selectedFileName = this.selectedFile?.name || "กรุณาเลือกไฟล์"
  }

  uploadFile() {
    if (!this.selectedFile) {
      alert('กรุณาเลือกไฟล์ก่อนอัปโหลด')
      return
    }
    const formData = new FormData();
    formData.append('file', this.selectedFile);
    this.bu6.loading = true
    this.fileService.uploadExcel(formData, 'mbu6').subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getBu6List()
        } else {
          this.showAlert(response.message, 'error')
          this.bu6.loading = false
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.bu6.loading = false
      }
    })
  }

  downloadFile() {
    const fileName = 'IMPORT_BU.xlsx'
    this.fileService.downloadTemplate(fileName).subscribe({
      next: response => {
        const url = window.URL.createObjectURL(response);
        const a = document.createElement("a");
        a.href = url;
        a.download = fileName;
        document.body.appendChild(a);
        a.click();
        document.body.removeChild(a);
        window.URL.revokeObjectURL(url);
      }, error: error => {
        this.showAlert(error.message, 'error')
      }
    })

  }

  getBu6List() {
    this.bu6.loading = true
    this.selectedItems.data.clear()
    this.bu6Service.getList().subscribe({
      next: response => {
        this.bu6.dataList = response.map(x => {
          this.selectedItems.data.set(x.bu6id, false)
          return new MyBu6Model(x)
        })
        this.selectedItems.key = 'bu6id'
        this.selectedItems.count = 0
        this.bu6.loading = false
        this.cdr.detectChanges()
      }, error: error => {
        this.bu6.loading = false
        this.cdr.detectChanges()
      }
    })
  }

  selectBu6(bu6?: Bu6Model) {
    if (bu6) {
      this.bu6.select = new MyBu6Model(bu6)
    } else if (this.currentModal == 'add') {
      this.bu6.select = new MyBu6Model()
    } else if (this.currentModal == 'edit') {
      this.bu6.select = new MyBu6Model({ bu6id: this.bu6.select.bu6id })
    }
    this.selectBu5()
    if (this.bu6.select.parent) {
      this.bu5Service.getById(this.bu6.select.parent).subscribe(response => {
        this.bu5 = new MyBu5Model(response)
        this.cdr.detectChanges()
      })
    }
  }

  addBu6() {
    this.bu6.loading = true
    this.bu6Service.post({ ...this.bu6.select, parent: this.bu5.bu5id }).subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getBu6List()
        } else {
          this.showAlert(response.message, 'error')
          this.bu6.loading = false
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.bu6.loading = false
      }
    })
  }
  deleteBu6() {
    this.bu6.loading = true
    const selectedKeys = Array.from(this.selectedItems.data.keys());
    const body = this.bu6.dataList.filter(x => selectedKeys.includes(x.bu6id) && this.selectedItems.data.get(x.bu6id)).map(x => new MyBu6Model(x))
    this.bu6Service.delete(body).subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getBu6List()
        } else {
          this.showAlert(response.message, 'error')
          this.bu6.loading = false
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.bu6.loading = false
      }
    })
  }

  getBu5List() {
    this.bu5Service.getList().subscribe(response => {
      this.bu5List = response.map(x => new MyBu5Model(x))
      this.onBu5ModalSearchChange()
    })
  }
  onBu5ModalSearchChange() {
    this.bu5Modal.currentPage = 1
    this.bu5Modal.page = Array.from({ length: Math.ceil(this.filterBu5Modal().length / 10) }, (_, i) => i + 1);
  }
  bu5idChange() {
    const bu5 = this.bu5List.find(x => x.bu5id == this.bu5.bu5id)
    this.selectBu5(bu5 || new MyBu5Model({ bu5id: this.bu5.bu5id }))
  }
  filterBu5Modal() {
    return this.bu5List.filter(x => x.bu5id.toLowerCase().includes(this.bu5Modal.search.toLowerCase()) ||
      x.tdesc.toLowerCase().includes(this.bu5Modal.search.toLowerCase()) ||
      x.edesc.toLowerCase().includes(this.bu5Modal.search.toLowerCase()))
  }
  selectBu5(bu5?: Bu5Model) {
    this.bu5 = new MyBu5Model(bu5)
  }

  showAlert(text: string, type: 'success' | 'error') {
    this.toastr[type](text, 'แจ้งเตือน', {
      timeOut: 3000,
      positionClass: 'toast-top-right',
    })
  }

  checkPrimary() {
    return this.bu6.dataList.find(x => x.bu6id == this.bu6.select.bu6id)
  }

  numSelectItem() {
    const selectedKeys = Array.from(this.selectedItems.data.keys());
    const num = this.bu6.dataList.filter(x => selectedKeys.includes(x.bu6id) && this.selectedItems.data.get(x.bu6id)).length
    return num
  }

  onSelectItemChange(arg: any) {
    this.selectedItems = arg
  }
}


