import { ChangeDetectorRef, Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { ColumnModel } from '@syncfusion/ej2-grids';
import { ToastrService } from 'ngx-toastr';
import { MyPLModel, PLModel } from 'src/app/shared/model/pl.model';
import { FileService } from 'src/app/shared/services/file.service';
import { PLService } from 'src/app/shared/services/pl.service';
@Component({
  selector: 'app-employee-level',
  templateUrl: './employee-level.component.html',
  styleUrls: ['./employee-level.component.scss']
})
export class EmployeeLevel implements OnInit {
  search = ""
  currentModal: 'add' | 'edit' | 'delete' = 'add'
  pl: { loading: boolean, select: PLModel, dataList: PLModel[] } = { loading: false, select: new MyPLModel(), dataList: [] }


  selectedFile: File | null = null;
  selectedFileName: string = 'กรุณาเลือกไฟล์';


  columns: ColumnModel[] = [{
    field: "plId",
    headerText: "รหัสระดับพนักงาน",
    type: "string",
    isPrimaryKey: true,
  },
  {
    field: "tdesc",
    headerText: "รายละเอียดระดับพนักงาน (ไทย)",
    type: "string"
  },
  {
    field: "edesc",
    headerText: "รายละเอียดระดับพนักงาน (อังกฤษ)",
    type: "string"
  }]
  searchSettings = {
    fields: ['plId', 'tdesc', 'edesc'],
    operator: 'contains',
    ignoreCase: false
  }
  selectedItems: { key: string, count: number, data: Map<string, boolean> } = { key: '', count: 0, data: new Map<string, boolean>() };
  constructor(private plService: PLService,
    private toastr: ToastrService,
    private cdr: ChangeDetectorRef,
    private fileService: FileService
  ) { }

  ngOnInit(): void {
    this.getPLList()
  }

  onFileSelected(event: any) {
    this.selectedFile = event.target.files.length > 0 ? event.target.files[0] : null;
    this.selectedFileName = this.selectedFile?.name || "กรุณาเลือกไฟล์"
  }

  uploadFile() {
    if (!this.selectedFile) {
      alert('กรุณาเลือกไฟล์ก่อนอัปโหลด')
      return
    }
    const formData = new FormData();
    formData.append('file', this.selectedFile);
    this.pl.loading = true
    this.fileService.uploadExcel(formData, 'pl').subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getPLList()
        } else {
          this.showAlert(response.message, 'error')
          this.pl.loading = false
          this.cdr.detectChanges()
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.pl.loading = false
        this.cdr.detectChanges()
      }
    })
  }

  downloadFile() {
    const fileName = 'IMPORT_PL.xlsx'
    this.fileService.downloadTemplate(fileName).subscribe({
      next: response => {
        const url = window.URL.createObjectURL(response);
        const a = document.createElement("a");
        a.href = url;
        a.download = fileName;
        document.body.appendChild(a);
        a.click();
        document.body.removeChild(a);
        window.URL.revokeObjectURL(url);
      }, error: error => {
        this.showAlert(error.message, 'error')
      }
    })

  }

  getPLList() {
    this.pl.loading = true
    this.selectedItems.data.clear()
    this.plService.getList().subscribe({
      next: response => {
        this.pl.dataList = response.map(x => {
          this.selectedItems.data.set(x.plId, false)
          return new MyPLModel(x)
        })
        this.selectedItems.key = 'plId'
        this.selectedItems.count = 0
        this.pl.loading = false
        this.cdr.detectChanges();
      }, error: error => {
        this.pl.loading = false
        this.cdr.detectChanges()
      }
    });
  }

  selectPl(pl?: PLModel) {
    if (pl) {
      this.pl.select = new MyPLModel(pl)
    } else if (this.currentModal == 'add') {
      this.pl.select = new MyPLModel()
    } else if (this.currentModal == 'edit') {
      this.pl.select = new MyPLModel({ plId: this.pl.select.plId })
    }
  }

  addPL() {
    this.pl.loading = true
    this.plService.post(this.pl.select).subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getPLList()
        } else {
          this.showAlert(response.message, 'error')
          this.pl.loading = true
          this.cdr.detectChanges()
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.pl.loading = true
        this.cdr.detectChanges()
      }
    })
  }

  deletePL() {
    this.pl.loading = true
    const selectedKeys = Array.from(this.selectedItems.data.keys());
    const body = this.pl.dataList.filter(x => selectedKeys.includes(x.plId) && this.selectedItems.data.get(x.plId)).map(x => new MyPLModel(x))
    this.plService.delete(body).subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getPLList()
        } else {
          this.showAlert(response.message, 'error')
          this.pl.loading = true
          this.cdr.detectChanges()
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.pl.loading = true
        this.cdr.detectChanges()
      }
    })
  }

  showAlert(text: string, type: 'success' | 'error') {
    this.toastr[type](text, 'แจ้งเตือน', {
      timeOut: 3000,
      positionClass: 'toast-top-right',
    });
  }


  checkPrimary() {
    return this.pl.dataList.find(x => x.plId == this.pl.select.plId)
  }

  numSelectItem() {
    const selectedKeys = Array.from(this.selectedItems.data.keys());
    const num = this.pl.dataList.filter(x => selectedKeys.includes(x.plId) && this.selectedItems.data.get(x.plId)).length
    return num
  }

  onSelectItemChange(arg: any) {
    this.selectedItems = arg
  }
}
