import { ChangeDetectorRef, Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { ColumnModel } from '@syncfusion/ej2-grids';
import { ToastrService } from 'ngx-toastr';
import { MyPositionModel, PositionModel } from 'src/app/shared/model/position.model';
import { FileService } from 'src/app/shared/services/file.service';
import { PositionService } from 'src/app/shared/services/position.service';
@Component({
  selector: 'app-position-unit',
  templateUrl: './position-unit.component.html',
  styleUrls: ['./position-unit.component.scss']
})
export class PositionUnitComponent implements OnInit {
  selectedFile: File | null = null;
  selectedFileName: string = 'กรุณาเลือกไฟล์';
  currentModal: 'add' | 'edit' | 'delete' = 'add'
  position: { loading: boolean, select: PositionModel, dataList: PositionModel[] } = { loading: false, select: new MyPositionModel(), dataList: [] }

  columns: ColumnModel[] = [{
    field: "positionId",
    headerText: "รหัสตำเเหน่ง",
    type: "string",
    isPrimaryKey: true,
  },
  {
    field: "tdesc",
    headerText: "รายละเอียดตำเเหน่ง (ไทย)",
    type: "string"
  },
  {
    field: "edesc",
    headerText: "รายละเอียดตำเเหน่ง (อังกฤษ)",
    type: "string"
  }]
  searchSettings = {
    fields: ['positionId', 'tdesc', 'edesc'],
    operator: 'contains',
    ignoreCase: false
  };
  search = ''
  selectedItems: { key: string, count: number, data: Map<string, boolean> } = { key: '', count: 0, data: new Map<string, boolean>() };
  constructor(private positionService: PositionService,
    private toastr: ToastrService,
    private cdr: ChangeDetectorRef,
    private fileService: FileService
  ) { }
  ngOnInit(): void {
    this.getPositionList()
  }

  onFileSelected(event: any) {
    this.selectedFile = event.target.files.length > 0 ? event.target.files[0] : null;
    this.selectedFileName = this.selectedFile?.name || "กรุณาเลือกไฟล์"
  }

  uploadFile() {
    if (!this.selectedFile) {
      alert('กรุณาเลือกไฟล์ก่อนอัปโหลด')
      return
    }
    const formData = new FormData();
    formData.append('file', this.selectedFile);
    this.position.loading = true
    this.fileService.uploadExcel(formData, 'mposition').subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getPositionList()
        } else {
          this.showAlert(response.message, 'error')
          this.position.loading = false
          this.cdr.detectChanges();
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.position.loading = false
        this.cdr.detectChanges();
      }
    })
  }

  downloadFile() {
    const fileName = 'IMPORT_MPOSITION.xlsx'
    this.fileService.downloadTemplate(fileName).subscribe({
      next: response => {
        const url = window.URL.createObjectURL(response);
        const a = document.createElement("a");
        a.href = url;
        a.download = fileName;
        document.body.appendChild(a);
        a.click();
        document.body.removeChild(a);
        window.URL.revokeObjectURL(url);
      }, error: error => {
        this.showAlert(error.message, 'error')
      }
    })

  }
  getPositionList() {
    this.position.loading = true
    this.selectedItems.data.clear()
    this.positionService.getList().subscribe({
      next: response => {
        this.position.dataList = response.map(x => {
          this.selectedItems.data.set(x.positionId, false)
          return new MyPositionModel(x)
        })
        this.selectedItems.key = 'positionId'
        this.selectedItems.count = 0
        this.position.loading = false
        this.cdr.detectChanges();
      }, error: error => {
        this.position.loading = false
        console.error('Error fetching employee types:', error);
        this.cdr.detectChanges()
      }
    })
  }

  selectPosition(position?: PositionModel) {
    if (position) {
      this.position.select = new MyPositionModel(position)
    } else if (this.currentModal == 'add') {
      this.position.select = new MyPositionModel()
    } else if (this.currentModal == 'edit') {
      this.position.select = new MyPositionModel({ positionId: this.position.select.positionId })
    }
  }

  addPosition() {
    this.position.loading = true
    this.positionService.post(this.position.select).subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getPositionList()
        } else {
          this.showAlert(response.message, 'error')
          this.position.loading = false
          this.cdr.detectChanges()
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.position.loading = false
        this.cdr.detectChanges()
      }
    })
  }
  deletePosition() {
    this.position.loading = true
    const selectedKeys = Array.from(this.selectedItems.data.keys());
    const body = this.position.dataList.filter(x => selectedKeys.includes(x.positionId) && this.selectedItems.data.get(x.positionId)).map(x => new MyPositionModel(x))
    this.positionService.delete(body).subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getPositionList()
        } else {
          this.showAlert(response.message, 'error')
          this.position.loading = false
          this.cdr.detectChanges()
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.position.loading = false
        this.cdr.detectChanges()
      }
    })
  }
  showAlert(text: string, type: 'success' | 'error') {
    this.toastr[type](text, 'แจ้งเตือน', {
      timeOut: 3000,
      positionClass: 'toast-top-right',
    });
  }



  checkPrimary() {
    return this.position.dataList.find(x => x.positionId == this.position.select.positionId)
  }

  numSelectItem() {
    const selectedKeys = Array.from(this.selectedItems.data.keys());
    const num = this.position.dataList.filter(x => selectedKeys.includes(x.positionId) && this.selectedItems.data.get(x.positionId)).length
    return num
  }

  onSelectItemChange(arg: any) {
    this.selectedItems = arg
  }
}

