import { ChangeDetectorRef, Component, EventEmitter, Input, Output } from '@angular/core';
import { ToastrService } from 'ngx-toastr';
import { CompetencyCourseModel, MyCompetencyCourseModel } from 'src/app/shared/model/competency-course.model';
import { MyCompetencycourseModel } from 'src/app/shared/model/competencycourse.model';
import { CompetencycourseService } from 'src/app/shared/services/competencycourse.service';
import { FileService } from 'src/app/shared/services/file.service';
@Component({
  selector: 'app-development-course',
  templateUrl: './development-course.component.html',
  styleUrls: ['./development-course.component.scss']
})
export class DevelopmentCourseComponent {
  currentPage = 1
  page = Array.from({ length: 1 }, (_, i) => i + 1);
  search = ""

  currentModal: 'add' | 'edit' | 'delete' | 'deleteGroup' = "add";
  competencyCourse: { loading: boolean, select: CompetencyCourseModel, dataList: { check: boolean, data: CompetencyCourseModel }[] } = { loading: false, select: new MyCompetencyCourseModel(), dataList: [] }
  numDataListChecked = 0
  isDataListChecked = false
  isDataListCheckedAll = false

  selectedFile: File | null = null;
  selectedFileName: string = 'กรุณาเลือกไฟล์';
  constructor(private toastr: ToastrService,
    private cdr: ChangeDetectorRef,
    private competencycourseService: CompetencycourseService,
    private fileService: FileService
  ) { }
  ngOnInit(): void {
    this.getCompetencycourseList()
  }
  onFileSelected(event: any) {
    this.selectedFile = event.target.files.length > 0 ? event.target.files[0] : null;
    this.selectedFileName = this.selectedFile?.name || "กรุณาเลือกไฟล์"
  }
  uploadFile() {
    if (!this.selectedFile) {
      alert('กรุณาเลือกไฟล์ก่อนอัปโหลด')
      return
    }
    const formData = new FormData();
    formData.append('file', this.selectedFile);
    this.competencyCourse.loading = true
    this.fileService.uploadExcel(formData, 'COMPETENCY_COURSE').subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getCompetencycourseList()
        } else {
          this.showAlert(response.message, 'error')
          this.competencyCourse.loading = false
          this.cdr.detectChanges()
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.competencyCourse.loading = false
        this.cdr.detectChanges()
      }
    })
  }
  downloadFile() {
    const fileName = 'IMPORT_COMPETENCY_COURSE.xlsx'
    this.fileService.downloadTemplate(fileName).subscribe({
      next: response => {
        const url = window.URL.createObjectURL(response);
        const a = document.createElement("a");
        a.href = url;
        a.download = fileName;
        document.body.appendChild(a);
        a.click();
        document.body.removeChild(a);
        window.URL.revokeObjectURL(url);
      }, error: error => {
        this.showAlert(error.message, 'error')
      }
    })

  }

  getCompetencycourseList() {
    this.competencyCourse.loading = true
    this.competencycourseService.getList().subscribe({
      next: response => {
        this.competencyCourse.dataList = response.map(x => ({ check: false, data: new MyCompetencyCourseModel(x) }))
        this.competencyCourse.loading = false
        this.isDataListCheckedAll = false
        this.dataListCheckAll()
        this.searchChange()
        this.cdr.detectChanges()
      }, error: error => {
        this.competencyCourse.loading = false
        this.cdr.detectChanges()
      }
    })
  }
  searchChange() {
    this.currentPage = 1
    this.page = Array.from({ length: Math.ceil(this.dataListFilter().length / 10) }, (_, i) => i + 1);
    this.dataListCheck()
  }
  dataListFilter() {
    return this.competencyCourse.dataList.filter(x => {
      const data = x.data
      const match = data.competencyCourseId.toLowerCase().includes(this.search.toLowerCase()) || data.tdesc.toLowerCase().includes(this.search.toLowerCase())
      return match;
    });
  }
  selectCompetencycourse(data?: CompetencyCourseModel) {
    this.competencyCourse.select = new MyCompetencyCourseModel(data)
  }

  updateCompetencyCourse(typeApi: 'post' | 'delete') {
    this.competencyCourse.loading = true
    let body: CompetencyCourseModel | CompetencyCourseModel[]
    switch (this.currentModal) {
      case ('deleteGroup'): {
        body = this.competencyCourse.dataList.filter(x => x.check).map(x => new MyCompetencyCourseModel(x.data))
        break;
      }
      default: {
        body = new MyCompetencyCourseModel(this.competencyCourse.select)
      }
    }
    this.competencycourseService[typeApi]((body as any)).subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getCompetencycourseList()
          this.searchChange()
        } else {
          this.showAlert(response.message, 'error')
          this.competencyCourse.loading = false
        }
        this.cdr.detectChanges()
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.competencyCourse.loading = false
        this.cdr.detectChanges()
      }
    })
  }
  showAlert(text: string, type: 'success' | 'error') {
    this.toastr[type](text, 'แจ้งเตือน', {
      timeOut: 3000,
      positionClass: 'toast-top-right',
    })
  }

  dataListCheckAll() {
    const selectAll = this.isDataListCheckedAll;
    this.competencyCourse.dataList.filter(x => {
      const data = x.data
      const match = data.competencyCourseId.toLowerCase().includes(this.search.toLowerCase()) || data.tdesc.toLowerCase().includes(this.search.toLowerCase())
      return match;
    }).forEach(x => x.check = selectAll);
    this.dataListCheck();
  }

  dataListCheck() {
    const dataCheck = this.dataListFilter();
    this.isDataListCheckedAll = dataCheck.length ? dataCheck.every(x => x.check) : false;
    this.numDataListChecked = this.competencyCourse.dataList.filter(x => x.check).length;
    this.isDataListChecked = Boolean(this.numDataListChecked)
  }

  clearData(modalStatus: string) {
    if (modalStatus == 'add') {
      this.selectCompetencycourse()
    } else if (modalStatus == 'edit') {
      this.selectCompetencycourse(new MyCompetencyCourseModel({ competencyCourseId: this.competencyCourse.select.competencyCourseId }))
    }
  }

  checkPrimary() {
    return this.competencyCourse.dataList.find(x => x.data.competencyCourseId == this.competencyCourse.select.competencyCourseId)
  }
}
