import { ChangeDetectorRef, Component, EventEmitter, Input, Output } from '@angular/core';
import { ToastrService } from 'ngx-toastr';
import { AssessmentModel, MyAssessmentModel } from 'src/app/shared/model/assessment.model';
import { AssessmentService } from 'src/app/shared/services/assessment.service';
import { FileService } from 'src/app/shared/services/file.service';
export interface DataModel {
  id: string
  name: string
  edesc: string
  code: string
  checked: boolean
}
@Component({
  selector: 'app-assessment-tool',
  templateUrl: './assessment-tool.component.html',
  styleUrls: ['./assessment-tool.component.scss']
})
export class AssessmentToolComponent {
  currentPage = 1
  page = Array.from({ length: 1 }, (_, i) => i + 1);
  search = ""
  numDataListChecked = 0
  isDataListChecked = false
  isDataListCheckedAll = false

  assessment: { loading: boolean, select: AssessmentModel, dataList: { check: boolean, data: AssessmentModel }[] } = { loading: false, select: new MyAssessmentModel(), dataList: [] }

  currentModal: 'add' | 'edit' | 'delete' | 'deleteGroup' = "add"
  selectedFile: File | null = null;
  selectedFileName: string = 'กรุณาเลือกไฟล์';
  constructor(private toastr: ToastrService,
    private cdr: ChangeDetectorRef,
    private assessmentService: AssessmentService,
    private fileService: FileService
  ) { }
  ngOnInit(): void {
    this.getAssessmentList()
  }
  onFileSelected(event: any) {
    this.selectedFile = event.target.files.length > 0 ? event.target.files[0] : null;
    this.selectedFileName = this.selectedFile?.name || "กรุณาเลือกไฟล์"
  }
  uploadFile() {
    if (!this.selectedFile) {
      alert('กรุณาเลือกไฟล์ก่อนอัปโหลด')
      return
    }
    const formData = new FormData();
    formData.append('file', this.selectedFile);
    this.assessment.loading = true
    this.fileService.uploadExcel(formData, 'COMPETENCY_ASSESSMENT').subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getAssessmentList()
        } else {
          this.showAlert(response.message, 'error')
          this.assessment.loading = false
          this.cdr.detectChanges()
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.assessment.loading = false
        this.cdr.detectChanges()
      }
    })
  }
  downloadFile() {
    const fileName = 'IMPORT_COMPETENCY_ASSESSMENT.xlsx'
    this.fileService.downloadTemplate(fileName).subscribe({
      next: response => {
        const url = window.URL.createObjectURL(response);
        const a = document.createElement("a");
        a.href = url;
        a.download = fileName;
        document.body.appendChild(a);
        a.click();
        document.body.removeChild(a);
        window.URL.revokeObjectURL(url);
      }, error: error => {
        this.showAlert(error.message, 'error')
      }
    })

  }
  getAssessmentList() {
    this.assessment.loading = true
    this.assessmentService.getList().subscribe({
      next: response => {
        this.assessment.dataList = response.map(x => ({ check: false, data: new MyAssessmentModel(x) }))
        this.isDataListCheckedAll = false
        this.dataListCheckAll()
        this.assessment.loading = false
        this.searchChange()
        this.cdr.detectChanges()
      }, error: error => {
        this.assessment.loading = false
        this.cdr.detectChanges()
      }
    })
  }
  assessmentListFilter() {
    return this.assessment.dataList.filter(x => {
      const data = x.data
      const match = data.assessmentId.toLowerCase().includes(this.search.toLowerCase()) ||
        data.tdesc.toLowerCase().includes(this.search.toLowerCase()) ||
        data.shortName.toLowerCase().includes(this.search.toLowerCase())
      return match
    })
  }
  selectAssessment(data?: AssessmentModel) {
    this.assessment.select = new MyAssessmentModel(data)
  }
  clearAssessment(currentModal: string) {
    if (currentModal == 'add') {
      this.selectAssessment()
    } else if (currentModal == 'edit') {
      this.selectAssessment(new MyAssessmentModel({ assessmentId: this.assessment.select.assessmentId }))
    }
  }
  searchChange() {
    this.currentPage = 1
    this.page = Array.from({ length: Math.ceil(this.assessmentListFilter().length / 10) }, (_, i) => i + 1);
    this.dataListCheck()
  }
  showAlert(text: string, type: 'success' | 'error') {
    this.toastr[type](text, 'แจ้งเตือน', {
      timeOut: 3000,
      positionClass: 'toast-top-right',
    })
  }
  updateAssessment(typeApi: 'post' | 'delete') {
    this.assessment.loading = true
    let body: AssessmentModel | AssessmentModel[]
    switch (this.currentModal) {
      case ('delete'): {
        body = new MyAssessmentModel(this.assessment.select)
        break;
      }
      case ('deleteGroup'): {
        body = this.assessment.dataList.filter(x => x.check).map(x => new MyAssessmentModel(x.data))
        break;
      }
      default: {
        body = new MyAssessmentModel(this.assessment.select)
      }
    }
    this.assessmentService[typeApi]((body as any)).subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getAssessmentList()
          this.searchChange()
        } else {
          this.showAlert(response.message, 'error')
          this.assessment.loading = false
        }
        this.cdr.detectChanges()
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.assessment.loading = false
        this.cdr.detectChanges()
      }
    })
  }
  dataListCheckAll() {
    const selectAll = this.isDataListCheckedAll;
    this.assessment.dataList.filter(x => {
      const data = x.data
      const match = data.assessmentId.toLowerCase().includes(this.search.toLowerCase()) ||
        data.tdesc.toLowerCase().includes(this.search.toLowerCase()) ||
        data.shortName.toLowerCase().includes(this.search.toLowerCase())
      return match
    }).forEach(x => x.check = selectAll);
    this.dataListCheck();
  }
  dataListCheck() {
    const dataCheck = this.assessmentListFilter();
    this.isDataListCheckedAll = dataCheck.length ? dataCheck.every(x => x.check) : false;
    this.numDataListChecked = this.assessment.dataList.filter(x => x.check).length;
    this.isDataListChecked = Boolean(this.numDataListChecked)
  }

  checkPrimary() {
    return this.assessment.dataList.find(x => x.data.assessmentId == this.assessment.select.assessmentId)
  }
}

