import { ChangeDetectorRef, Component, EventEmitter, Input, Output, ViewChild } from '@angular/core';
import { MatDialog } from '@angular/material/dialog';
import { CompetencycourseMiniModel, MyCompetencycourseMiniModel } from 'src/app/shared/model/competencycourse-mini.model';
import { IdpFormModel } from 'src/app/shared/model/idp-form.model';
import { CompetencycourseService } from 'src/app/shared/services/competencycourse.service';
import { EmployeeService } from 'src/app/shared/services/employee.service';
import { EvaluationIdpService } from 'src/app/shared/services/evaluation-Idp.service';
import Swal from 'sweetalert2';

interface table {
  currentPage: number,
  page: number[],
  search: string,
  pageSize: number
}
@Component({
  selector: 'app-idp-evalution',
  templateUrl: './idp-evalution.component.html',
  styleUrls: ['./idp-evalution.component.scss']
})
export class IdpEvalutionComponent {
  @Input() pathTitle: string[] = []
  @Input() evaluationRoundId = ""
  @Input() evaluateeId = ""
  @Input() evaluaterId = ""
  @Output() sendReturnPath: EventEmitter<any> = new EventEmitter<any>();
  currentDate = new Date()
  hoveredCode: string | null = null;
  formIdp: { loading: boolean, data?: IdpFormModel } = { loading: false, data: undefined }
  originalformIdp?: IdpFormModel
  canEdit = false
  canSave = false

  competencycourse: { loading: boolean, data: CompetencycourseMiniModel[] } = { loading: false, data: [] }
  competencycourseTable: table = {
    currentPage: 1,
    page: Array.from({ length: 1 }, (_, i) => i + 1),
    search: "",
    pageSize: 10
  }
  competencycourseIndex = -1
  @ViewChild("competencycourseModal") competencycourseModal: any;
  dialogRefCompetencycourseModal: any
  constructor(private evaluationIdpService: EvaluationIdpService,
    private employeeService: EmployeeService,
    private competencycourseService: CompetencycourseService,
    private cdr: ChangeDetectorRef,
    private dialog: MatDialog
  ) {
  }
  ngOnInit(): void {
    this.getFormIdp()
    this.getCompetencycourseMiniList()
  }
  getCompetencycourseMiniList() {
    this.competencycourse.loading = false
    this.competencycourseService.getMiniList().subscribe({
      next: response => {
        this.competencycourse.data = response.map(x => new MyCompetencycourseMiniModel(x))
        this.competencycourse.loading = false
        this.cdr.detectChanges()
      }, error: error => {
        this.competencycourse.loading = false
        this.cdr.detectChanges()
      }
    })
  }
  openCompetencycourseDialog(index: number) {
    this.competencycourseIndex = index
    this.dialogRefCompetencycourseModal = this.dialog.open(this.competencycourseModal, {
      width: '800px',
    })
  }
  closeCompetencycourseDialog() {
    this.dialogRefCompetencycourseModal.close()
  }

  addCompetencycourse(data: CompetencycourseMiniModel) {
    if (this.formIdp.data) {
      if (this.formIdp.data.competencyIndicatorsCourses1[this.competencycourseIndex].competencyCourse) {
        this.formIdp.data.competencyIndicatorsCourses1[this.competencycourseIndex].competencyCourse.push(new MyCompetencycourseMiniModel(data))
      } else {
        this.formIdp.data.competencyIndicatorsCourses1[this.competencycourseIndex]['competencyCourse'] = [new MyCompetencycourseMiniModel(data)]
      }
      this.cdr.detectChanges()
    }
  }

  deleteCompetencyCourse(index: number, removeIndex: number) {
    if (this.formIdp.data) {
      this.formIdp.data.competencyIndicatorsCourses1[index].competencyCourse.splice(removeIndex, 1)
    }
  }
  onCompetencycourseSearch() {
    this.competencycourseTable.currentPage = 1
    this.competencycourseTable.page = Array.from({ length: Math.ceil(this.filterCompetencycourse().length / 10) }, (_, i) => i + 1);
  }
  filterCompetencycourse() {
    return this.competencycourse.data.filter(x => x.competencyCourseId.toLowerCase().includes(this.competencycourseTable.search.toLowerCase()) ||
      x.tdesc.toLowerCase().includes(this.competencycourseTable.search.toLowerCase()) ||
      x.edesc.toLowerCase().includes(this.competencycourseTable.search.toLowerCase()))
  }
  resetFormIdp() {
    if (this.originalformIdp && this.formIdp.data) {
      this.formIdp.data.competencyIndicatorsCourses1 = JSON.parse(JSON.stringify(this.originalformIdp.competencyIndicatorsCourses1))
    }
  }
  getFormIdp() {
    this.formIdp.loading = true
    this.evaluationIdpService.getFormIdp(this.evaluationRoundId, this.evaluateeId).subscribe({
      next: response => {
        this.formIdp.data = JSON.parse(JSON.stringify(response))
        this.originalformIdp = JSON.parse(JSON.stringify(response))
        if (this.formIdp.data) {
          this.canSave = (this.formIdp.data.currentStep == this.formIdp.data.checkStep) && (this.formIdp.data.masfromStatusType.code != 'Boss_finish' && this.formIdp.data.masfromStatusType.code != 'Not_evaluating_yet')
          this.canEdit = (this.formIdp.data.currentStep == this.formIdp.data.checkStep && (+(this.formIdp.data.checkStep) <= 1)) && (this.formIdp.data.masfromStatusType.code != 'Boss_finish' && this.formIdp.data.masfromStatusType.code != 'Not_evaluating_yet')
        }
        switch (this.formIdp.data?.currentStep) {
          case ("0"): {
            if (this.canEdit) {
              this.formIdp.data.masfromEvaluationIdp.apsassessyDate = this.currentDate.toISOString().split('T')[0];
            }
            break
          }
          case ("1"): {
            if (this.canEdit) {
              this.formIdp.data.masfromEvaluationIdp.apsapprove1Date = this.currentDate.toISOString().split('T')[0];
            }
            break
          }
          case ("2"): {
            if (this.canEdit) {
              this.formIdp.data.masfromEvaluationIdp.apsapprove2Date = this.currentDate.toISOString().split('T')[0];
            }
            break
          }
          case ("3"): {
            if (this.canEdit) {
              this.formIdp.data.masfromEvaluationIdp.apsapprove3Date = this.currentDate.toISOString().split('T')[0];
            }
            break
          }
          case ("4"): {
            if (this.canEdit) {
              this.formIdp.data.masfromEvaluationIdp.apsapprove4Date = this.currentDate.toISOString().split('T')[0];
            }
            break
          }
          case ("5"): {
            if (this.canEdit) {
              this.formIdp.data.masfromEvaluationIdp.apsapprove5Date = this.currentDate.toISOString().split('T')[0];
            }
            break
          }
          default: { return }
        }
        this.formIdp.data.masfromEvaluationIdp.idpStatus = this.formIdp.data.masfromEvaluationIdp.idpStatus ? this.formIdp.data.masfromEvaluationIdp.idpStatus : '0'
        this.formIdp.loading = false
        this.cdr.detectChanges()
      }, error: error => {
        this.formIdp.loading = false
        this.cdr.detectChanges()
      }
    })
  }
  returnPath() {
    this.sendReturnPath.emit()
  }
  convertDate(dateInput?: string | Date): string {
    let date = new Date()
    if (dateInput) {
      if (typeof dateInput === 'string') {
        const [year, month, day] = dateInput.split('-').map(Number);
        date = new Date(year, month - 1, day);
      } else {
        date = dateInput
      }
    }
    return date?.toLocaleDateString('th-TH', { day: 'numeric', month: 'long', year: 'numeric' }) || ''
  }
  convertDateFormat(dateString: string): string {
    const [year, month, day] = dateString.split("-");
    return `${day}-${month}-${year}`;
  }


  save(status: 'approve' | 'noApprove' | 'draft') {
    let title = ''
    let text = ''
    let confirmButtonText = ''
    let approveStatus = '0'
    if (status == 'draft') {
      title = 'บันทึกแบบร่าง'
      text = 'คุณต้องการบันทึกแบบร่างของการประเมินนี้ใช่หรือไม่'
      confirmButtonText = 'ยืนยันการบันทึก'
      approveStatus = '1'
    } else if (status == 'approve') {
      approveStatus = '2'
      if (this.formIdp?.data?.checkStep == '0') {
        title = 'ยืนยันข้อมูล'
        text = 'คุณต้องการยืนยันข้อมูลการประเมินนี้ใช่หรือไม่'
        confirmButtonText = 'ยืนยันการบันทึก'
      } else {
        title == 'อนุมัติ'
        text = 'คุณต้องการอนุมัติข้อมูลการประเมินนี้ใช่หรือไม่'
        confirmButtonText = 'ยืนยันการอนุมัติ'
      }
    } else if (status == 'noApprove') {
      approveStatus = '0'
      title == 'ไม่อนุมัติ'
      text = 'คุณต้องการไม่อนุมัติข้อมูลการประเมินนี้ใช่หรือไม่'
      confirmButtonText = 'ยืนยันการไม่อนุมัติ'
    }
    Swal.fire({
      iconHtml: `
              <div class="flex items-center justify-center rounded-full !h-80px !w-80px" style="background-color: #E8F8EE;">
                  <svg width="39" height="39" viewBox="0 0 39 39" fill="none" xmlns="http://www.w3.org/2000/svg">
                      <rect width="39" height="39" fill="#D2D2D2"/>
                      <g id="Component">
                          <g id="alert cart">
                              <g id="mdi:file-export">
                                  <circle cx="22.5" cy="19.5" r="33.5" fill="#E8F8EE"/>
                                  <path d="M9.75 3.25C8.88805 3.25 8.0614 3.59241 7.4519 4.2019C6.84241 4.8114 6.5 5.63805 6.5 6.5V32.5C6.5 33.362 6.84241 34.1886 7.4519 34.7981C8.0614 35.4076 8.88805 35.75 9.75 35.75H29.25C30.112 35.75 30.9386 35.4076 31.5481 34.7981C32.1576 34.1886 32.5 33.362 32.5 32.5V13L22.75 3.25M21.125 5.6875L30.0625 14.625H21.125M14.5113 19.8575H26V31.3463L22.555 27.9013L17.9563 32.5L13.3575 27.9013L17.9563 23.3188" 
                                  fill="#1DBE5A"/>
                              </g>
                          </g>
                      </g>
                  </svg>
              </div>
              `,
      title: title,
      text: text,
      showCancelButton: true,
      confirmButtonText: confirmButtonText,
      cancelButtonText: 'ย้อนกลับ',
      customClass: {
        title: '!swal2-title-mt-20px',
        actions: '!swal2-actions-mt-20px',
        icon: '!swal2-icon-no-border',
        confirmButton: '!swal2-button-bg-green',
        cancelButton: '!swal2-button-bg-gray',
      },
    }).then((result) => {
      if (result.isConfirmed && this.formIdp) {
        this.saveApi(approveStatus)
      } else if (result.dismiss === Swal.DismissReason.cancel) {
        Swal.fire({
          title: 'ยกเลิก!',
          text: 'การบันทึกถูกยกเลิก',
          icon: 'error',
          customClass: {
            confirmButton: '!swal2-button-bg-danger',
          }
        })
      }
    })
  }

  saveApi(approveStatus: string) {
    let body = this.formIdp.data
    switch (this.formIdp.data?.currentStep) {
      case ("0"): {
        body = {
          ...this.formIdp.data, masfromEvaluationIdp: {
            ...this.formIdp.data.masfromEvaluationIdp,
            apsassessyStatus: approveStatus
          }
        }
        break
      }
      case ("1"): {
        body = {
          ...this.formIdp.data, masfromEvaluationIdp: {
            ...this.formIdp.data.masfromEvaluationIdp,
            apsapprove1Status: approveStatus
          }
        }
        break
      }
      case ("2"): {
        body = {
          ...this.formIdp.data, masfromEvaluationIdp: {
            ...this.formIdp.data.masfromEvaluationIdp,
            apsapprove1Status: approveStatus == '0' ? approveStatus : this.formIdp.data.masfromEvaluationIdp.apsapprove1Status,
            apsapprove2Status: approveStatus
          }
        }
        break
      }
      case ("3"): {
        body = {
          ...this.formIdp.data, masfromEvaluationIdp: {
            ...this.formIdp.data.masfromEvaluationIdp,
            apsapprove2Status: approveStatus == '0' ? approveStatus : this.formIdp.data.masfromEvaluationIdp.apsapprove2Status,
            apsapprove3Status: approveStatus
          }
        }
        break
      }
      case ("4"): {
        body = {
          ...this.formIdp.data, masfromEvaluationIdp: {
            ...this.formIdp.data.masfromEvaluationIdp,
            apsapprove3Status: approveStatus == '0' ? approveStatus : this.formIdp.data.masfromEvaluationIdp.apsapprove3Status,
            apsapprove4Status: approveStatus
          }
        }
        break
      }
      case ("5"): {
        body = {
          ...this.formIdp.data, masfromEvaluationIdp: {
            ...this.formIdp.data.masfromEvaluationIdp,
            apsapprove4Status: approveStatus == '0' ? approveStatus : this.formIdp.data.masfromEvaluationIdp.apsapprove4Status,
            apsapprove5Status: approveStatus
          }
        }
        break
      }
      default: { return }
    }
    this.evaluationIdpService.postIdp(body).subscribe({
      next: response => {
        if (response.success) {
          Swal.fire({
            title: 'บันทึกสำเร็จ!',
            text: 'การประเมินของคุณถูกบันทึกแล้ว',
            icon: 'success',
            customClass: {
              confirmButton: '!swal2-button-bg-green',
            }
          });
          this.getFormIdp()
        } else {
          Swal.fire({
            title: 'ยกเลิก!',
            text: 'การบันทึกถูกยกเลิก',
            icon: 'error',
            customClass: {
              confirmButton: '!swal2-button-bg-danger',
            }
          });
        }
      }, error: error => {
        Swal.fire({
          title: 'ยกเลิก!',
          text: 'การบันทึกถูกยกเลิก',
          icon: 'error',
          customClass: {
            confirmButton: '!swal2-button-bg-danger',
          }
        });
      }
    })
  }


}
