import { ChangeDetectorRef, Component, EventEmitter, Input, Output, SimpleChanges } from '@angular/core';
import { AppraisalPmsFormModel } from 'src/app/shared/model/appraisal-pms-form.model';
import { CompetencyGradeModel, MyCompetencyGradeModel } from 'src/app/shared/model/competency-grades.model';
import { AppraisalService } from 'src/app/shared/services/appraisal.service';
import { EmployeeService } from 'src/app/shared/services/employee.service';
import { PmsGroupGradeService } from 'src/app/shared/services/pms-group-grade.service';
import { SettingAssessmentService } from 'src/app/shared/services/setting-assessment.service';
import Swal from 'sweetalert2';

export interface Part8Model {
  id: number
  evaluationFactor: string,
  rawScore: number,
  factors: number
  scoreObtained: number
  netScore: number
}
export interface LevelStarModel {
  evaluationFactor: string,
  rawScore: number,
  factors: number
  scoreObtained: number
  netScore: number
}

@Component({
  selector: 'app-pms-evalution',
  templateUrl: './pms-evalution.component.html',
  styleUrls: ['./pms-evalution.component.scss']
})
export class PmsEvalutionComponent {
  part1show = true
  part2show = true
  part3show = true
  part4show = true
  part5show = true
  part6show = true
  part7show = true
  part8show = true
  data8List: Part8Model[] = [{
    id: 1,
    evaluationFactor: "Part 1 : ประเมินผลการปฏิบัติงานตามนโยบายบริษัท (Corporate KPI)",
    rawScore: 100,
    factors: 0,
    scoreObtained: 0,
    netScore: 0,
  }, {
    id: 2,
    evaluationFactor: "Part 2 : ประเมินผลการปฏิบัติงานประจำ (Department KPI)",
    rawScore: 100,
    factors: 0,
    scoreObtained: 0,
    netScore: 0,
  }, {
    id: 3,
    evaluationFactor: "Part 3 : ประเมินผลการปฏิบัติงานประจำ (Individual KPI)",
    rawScore: 100,
    factors: 0,
    scoreObtained: 0,
    netScore: 0,
  }, {
    id: 4,
    evaluationFactor: "Part 4 : ประเมินผลสมรรถนะที่สนับสนุนการปฏิบัติงาน (Competency)",
    rawScore: 100,
    factors: 0,
    scoreObtained: 0,
    netScore: 0,
  }, {
    id: 5,
    evaluationFactor: "Part 5 : อัตราการเข้างาน (Time Attendance)",
    rawScore: 100,
    factors: 0,
    scoreObtained: 0,
    netScore: 0,
  }, {
    id: 6,
    evaluationFactor: "Part 6 : งานที่ได้รับมอบหมายเพิ่มเติม (Cross Functional Project Assignment)",
    rawScore: 100,
    factors: 0,
    scoreObtained: 0,
    netScore: 0,
  }, {
    id: 7,
    evaluationFactor: "Part 7 : กิจกรรมพิเศษ (Special Activities)",
    rawScore: 100,
    factors: 0,
    scoreObtained: 0,
    netScore: 0,
  }]
  part9show = true
  part10show = true
  canEdit = false
  canSave = false
  currentDate = new Date()
  @Input() pathTitle: string[] = []
  @Input() evaluationRoundId = ""
  @Input() evaluateeId = ""
  @Input() evaluaterId = ""
  @Input() competencyTypeId = ""
  @Output() sendReturnPath: EventEmitter<any> = new EventEmitter<any>();
  appraisalPms: { loading: boolean, data?: AppraisalPmsFormModel } = { loading: false, data: undefined }
  groupGrade: { loading: boolean, dataList: CompetencyGradeModel[] } = { loading: false, dataList: [] }
  constructor(private appraisalService: AppraisalService,
    private pmsGroupGradeService: PmsGroupGradeService,
    private employeeService: EmployeeService,
    private cdr: ChangeDetectorRef,
    private settingAssessmentService: SettingAssessmentService
  ) {
  }
  ngOnChanges(changes: SimpleChanges): void {
    if (changes['evaluateeId'].currentValue || changes['evaluaterId'].currentValue) {
      this.toggleAllParts(this.evaluateeId == this.evaluaterId)
    }
  }
  ngOnInit(): void {
    this.getPmsGroupGradeList()
    this.getAppraisalPmsForm()
  }
  getPmsGroupGradeList() {
    this.groupGrade.loading = true
    this.pmsGroupGradeService.getGradeList().subscribe({
      next: response => {
        this.groupGrade.dataList = response.map(x => new MyCompetencyGradeModel(x))
        this.groupGrade.loading = false
        this.cdr.detectChanges()
      }, error: error => {
        this.groupGrade.loading = false
        this.cdr.detectChanges()
      }
    })
  }
  getAppraisalPmsForm() {
    this.appraisalPms.loading = true
    this.appraisalService.getFormPmsById(this.evaluationRoundId, this.evaluateeId).subscribe({
      next: response => {
        this.appraisalPms.data = JSON.parse(JSON.stringify(response))
        this.data8List.forEach((x, i) => {
          if (x.id == 1 && this.appraisalPms.data?.part1Percentage) {
            x.factors = this.appraisalPms.data.part1Percentage
          } else if (x.id == 2 && this.appraisalPms.data?.part2Percentage) {
            x.factors = this.appraisalPms.data.part2Percentage
          } else if (x.id == 3 && this.appraisalPms.data?.part3Percentage) {
            x.factors = this.appraisalPms.data.part3Percentage
          } else if (x.id == 4 && this.appraisalPms.data?.part4Percentage) {
            x.factors = this.appraisalPms.data.part4Percentage
          } else if (x.id == 5 && this.appraisalPms.data?.part5Percentage) {
            x.factors = this.appraisalPms.data.part5Percentage
          } else if (x.id == 6 && this.appraisalPms.data?.part6Percentage) {
            x.factors = this.appraisalPms.data.part6Percentage
          } else if (x.id == 7 && this.appraisalPms.data?.part7Percentage) {
            x.factors = this.appraisalPms.data.part7Percentage
          }
        })
        if (!this.appraisalPms.data?.part1Detail.length) {
          this.data8List = this.data8List.filter(item => item.id !== 1)
        }
        if (!this.appraisalPms.data?.part2Detail.length) {
          this.data8List = this.data8List.filter(item => item.id !== 2)
        }
        if (!this.appraisalPms.data?.part3Detail.length) {
          this.data8List = this.data8List.filter(item => item.id !== 3)
        }
        if (!this.appraisalPms.data?.part4Detail.length) {
          this.data8List = this.data8List.filter(item => item.id !== 4)
        }
        if (!this.appraisalPms.data?.part5Detail.length) {
          this.data8List = this.data8List.filter(item => item.id !== 5)
        }
        if (!this.appraisalPms.data?.part6Detail.length) {
          this.data8List = this.data8List.filter(item => item.id !== 6)
        }
        if (!this.appraisalPms.data?.part7Detail.length) {
          this.data8List = this.data8List.filter(item => item.id !== 7)
        }
        this.changePercentage()
        if (this.appraisalPms.data) {
          this.canSave = (this.appraisalPms.data.currentStep == this.appraisalPms.data.checkStep) && this.appraisalPms.data.masfromStatusType.code != 'Boss_finish' && this.appraisalPms.data.masfromStatusType.code != 'Not_evaluating_yet'
          this.canEdit = (this.appraisalPms.data.currentStep == this.appraisalPms.data.checkStep) && (+(this.appraisalPms.data.checkStep) <= 1) && this.appraisalPms.data.masfromStatusType.code != 'Boss_finish' && this.appraisalPms.data.masfromStatusType.code != 'Not_evaluating_yet'
          switch (this.appraisalPms.data?.currentStep) {
            case ("0"): {
              if (this.canSave) {
                this.appraisalPms.data.apsassessyDate = this.currentDate.toISOString().split('T')[0];
              }
              break
            }
            case ("1"): {
              if (this.canSave) {
                this.appraisalPms.data.apsapprove1Date = this.currentDate.toISOString().split('T')[0];
              }
              break
            }
            case ("2"): {
              if (this.canSave) {
                this.appraisalPms.data.apsapprove2Date = this.currentDate.toISOString().split('T')[0];
              }
              break
            }
            case ("3"): {
              if (this.canSave) {
                this.appraisalPms.data.apsapprove3Date = this.currentDate.toISOString().split('T')[0];
              }
              break
            }
            case ("4"): {
              if (this.canSave) {
                this.appraisalPms.data.apsapprove4Date = this.currentDate.toISOString().split('T')[0];
              }
              break
            }
            case ("5"): {
              if (this.canSave) {
                this.appraisalPms.data.apsapprove5Date = this.currentDate.toISOString().split('T')[0];
              }
              break
            }
            default: { return }
          }
        }
        this.appraisalPms.loading = false
        this.cdr.detectChanges()
      }, error: error => {
        this.appraisalPms.loading = false
        this.cdr.detectChanges()
      }
    })
  }
  returnPath() {
    this.sendReturnPath.emit()
  }

  calRawScore(data?: any) {
    if (data) {
      return data.reduce((sum: number, item: any) => sum + item.weight * 5, 0).toFixed(2);
    }
    return "0.00"
  }
  calWeigth(data?: any) {
    if (data) {
      return data.reduce((sum: number, item: any) => sum + item.weight, 0).toFixed(2);
    }
    return "0.00"
  }
  calExpectationScore(data?: any) {
    if (data) {
      return data.reduce((sum: number, item: any) => sum + item.scoreTopicExpectationBoss, 0).toFixed(2);
    }
    return "0.00"
  }
  calExpectationTotalScore(data?: any) {
    if (data) {
      return data.reduce((sum: number, item: any) => sum + (item.scoreTopicExpectationBoss * item.weight), 0).toFixed(2);
    }
    return "0.00"
  }
  calPercentage(A: number, B: number) {
    if (B > 0) {
      return ((A / B) * 100).toFixed(2);
    }
    return "0.00"
  }
  changePercentage() {
    this.data8List.forEach(x => {
      if (x.id == 1 && this.appraisalPms.data?.part1Detail) {
        x.scoreObtained = +this.calPercentage(this.calExpectationTotalScore(this.appraisalPms.data?.part1Detail), this.calRawScore(this.appraisalPms.data?.part1Detail))
        if (this.appraisalPms.data) {
          this.appraisalPms.data.part1SumScore = x.scoreObtained
        }
      } else if (x.id == 2 && this.appraisalPms.data?.part2Detail) {
        x.scoreObtained = +this.calPercentage(this.calExpectationTotalScore(this.appraisalPms.data?.part2Detail), this.calRawScore(this.appraisalPms.data?.part2Detail))
        if (this.appraisalPms.data) {
          this.appraisalPms.data.part2SumScore = x.scoreObtained
        }
      } else if (x.id == 3 && this.appraisalPms.data?.part3Detail) {
        x.scoreObtained = +this.calPercentage(this.calExpectationTotalScore(this.appraisalPms.data?.part3Detail), this.calRawScore(this.appraisalPms.data?.part3Detail))
        if (this.appraisalPms.data) {
          this.appraisalPms.data.part3SumScore = x.scoreObtained
        }
      } else if (x.id == 4 && this.appraisalPms.data?.part4Detail) {
        x.scoreObtained = +this.calPercentage(this.calExpectationTotalScorePart4(this.appraisalPms.data?.part4Detail), this.calRawScorePart4(this.appraisalPms.data?.part4Detail))
        if (this.appraisalPms.data) {
          this.appraisalPms.data.part4SumScore = x.scoreObtained
        }
      } else if (x.id == 5 && this.appraisalPms.data?.part5Detail) {
        x.scoreObtained = +this.calPercentage(this.calExpectationTotalScorePart5(this.appraisalPms.data?.part5Detail), this.calRawScorePart5(this.appraisalPms.data?.part5Detail))
        if (this.appraisalPms.data) {
          this.appraisalPms.data.part5SumScore = x.scoreObtained
        }
      } else if (x.id == 6 && this.appraisalPms.data?.part6Detail) {
        x.scoreObtained = +this.calPercentage(this.calExpectationTotalScore(this.appraisalPms.data?.part6Detail), this.calRawScore(this.appraisalPms.data?.part6Detail))
        if (this.appraisalPms.data) {
          this.appraisalPms.data.part6SumScore = x.scoreObtained
        }
      } else if (x.id == 7 && this.appraisalPms.data?.part7Detail) {
        x.scoreObtained = +this.calPercentage(this.calExpectationTotalScore(this.appraisalPms.data?.part7Detail), this.calRawScore(this.appraisalPms.data?.part7Detail))
        if (this.appraisalPms.data) {
          this.appraisalPms.data.part7SumScore = x.scoreObtained
        }
      }
      x.netScore = x.factors / 100 * x.scoreObtained
    })
    this.setGradeScore()
  }
  setGradeScore() {
    const gradeScore = this.groupGrade.dataList.find(item => Math.ceil(+this.calNetScore()) >= item.gradeMinScore && Math.ceil(+this.calNetScore()) <= item.gradeMaxScore);
    if (this.appraisalPms.data && gradeScore) {
      this.appraisalPms.data.gradeScore = gradeScore.gradeDetail
    }
  }

  calFactors() {
    return this.data8List.reduce((sum: number, item: any) => sum + item.factors, 0).toFixed(2);
  }
  calScoreObtained() {
    return this.data8List.reduce((sum: number, item: any) => sum + item.scoreObtained, 0).toFixed(2);
  }
  calNetScore() {
    if (this.appraisalPms.data) {
      this.appraisalPms.data.netScore = +(this.data8List.reduce((sum: number, item: any) => sum + item.netScore, 0).toFixed(2));
      return this.appraisalPms.data.netScore
    }
    return "0.00"
  }


  calAverage(score: number) {
    const roundedScore = score % 1 <= 0.5 ? Math.floor(score) : Math.round(score);
    if (roundedScore >= 90 && roundedScore <= 100) {
      return 5;
    } else if (roundedScore >= 80 && roundedScore <= 89) {
      return 4;
    } else if (roundedScore >= 60 && roundedScore <= 79) {
      return 3;
    } else if (roundedScore >= 40 && roundedScore <= 59) {
      return 2;
    } else {
      return 1;
    }
  }

  calRawScorePart4(data?: any) {
    if (data) {
      return data.reduce((sum: number, item: any) => sum + item.competencyType.weight * 5, 0).toFixed(2);
    }
    return "0.00"
  }

  calWeigthPart4(data?: any) {
    if (data) {
      return data.reduce((sum: number, item: any) => sum + item.competencyType.weight, 0).toFixed(2);
    }
    return "0.00"
  }
  calExpectationScorePart4(data?: any) {
    if (data) {
      return data.reduce((sum: number, item: any) => sum + this.calAverage(item.averageScore), 0).toFixed(2);
    }
    return "0.00"
  }
  calExpectationTotalScorePart4(data?: any) {
    if (data) {
      return data.reduce((sum: number, item: any) => sum + (item.competencyType.weight * this.calAverage(item.averageScore)), 0).toFixed(2);
    }
    return "0.00"
  }


  calRawScorePart5(data?: any) {
    if (data) {
      return data.reduce((sum: number, item: any) => sum + item.weight * 5, 0).toFixed(2);
    }
    return "0.00"
  }
  calWeigthPart5(data?: any) {
    if (data) {
      return data.reduce((sum: number, item: any) => sum + item.weight, 0).toFixed(2);
    }
    return "0.00"
  }
  calExpectationScorePart5(data?: any) {
    if (data) {
      return data.reduce((sum: number, item: any) => sum + item.pmsWorkingTimScoreBoss, 0).toFixed(2);
    }
    return "0.00"
  }
  calExpectationTotalScorePart5(data?: any) {
    if (data) {
      return data.reduce((sum: number, item: any) => sum + (item.pmsWorkingTimScoreBoss * item.weight), 0).toFixed(2);
    }
    return "0.00"
  }
  setScorePart5(item: any, score: number) {
    if (this.canEdit) {
      if (this.evaluaterId == this.evaluateeId) {
        item.pmsWorkingTimScore = score
        item.pmsWorkingTimScoreBoss = score
      } else {
        item.pmsWorkingTimScoreBoss = score
      }
      this.changePercentage()
    }
  }


  numberFixed2(data?: number) {
    if (data) {
      return data.toFixed(2)
    }
    return "0.00"
  }

  setScore(item: any, score: number) {
    if (this.canEdit) {
      if (this.evaluaterId == this.evaluateeId) {
        item.scoreTopicExpectation = score
        item.scoreTopicExpectationBoss = score
      } else {
        item.scoreTopicExpectationBoss = score
      }
      this.changePercentage()
    }
  }

  convertDate(dateInput?: string | Date): string {
    let date = new Date()
    if (dateInput) {
      if (typeof dateInput === 'string') {
        const [year, month, day] = dateInput.split('-').map(Number);
        date = new Date(year, month - 1, day);
      } else {
        date = dateInput
      }
    } else {
      return ''
    }
    return date?.toLocaleDateString('th-TH', { day: 'numeric', month: 'long', year: 'numeric' }) || ''
  }

  save(status: 'approve' | 'noApprove' | 'draft') {
    let title = ''
    let text = ''
    let confirmButtonText = ''
    let approveStatus = '0'
    if (status == 'draft') {
      title = 'บันทึกแบบร่าง'
      text = 'คุณต้องการบันทึกแบบร่างของการประเมินนี้ใช่หรือไม่'
      confirmButtonText = 'ยืนยันการบันทึก'
      approveStatus = '1'
    } else if (status == 'approve') {
      approveStatus = '2'
      if (this.appraisalPms?.data?.checkStep == '0') {
        title = 'ยืนยันข้อมูล'
        text = 'คุณต้องการยืนยันข้อมูลการประเมินนี้ใช่หรือไม่'
        confirmButtonText = 'ยืนยันการบันทึก'
      } else {
        title == 'อนุมัติ'
        text = 'คุณต้องการอนุมัติข้อมูลการประเมินนี้ใช่หรือไม่'
        confirmButtonText = 'ยืนยันการอนุมัติ'
      }
    } else if (status == 'noApprove') {
      approveStatus = '0'
      title == 'ไม่อนุมัติ'
      text = 'คุณต้องการไม่อนุมัติข้อมูลการประเมินนี้ใช่หรือไม่'
      confirmButtonText = 'ยืนยันการไม่อนุมัติ'
    }
    Swal.fire({
      iconHtml: `
              <div class="flex items-center justify-center rounded-full !h-80px !w-80px" style="background-color: #E8F8EE;">
                  <svg width="39" height="39" viewBox="0 0 39 39" fill="none" xmlns="http://www.w3.org/2000/svg">
                      <rect width="39" height="39" fill="#D2D2D2"/>
                      <g id="Component">
                          <g id="alert cart">
                              <g id="mdi:file-export">
                                  <circle cx="22.5" cy="19.5" r="33.5" fill="#E8F8EE"/>
                                  <path d="M9.75 3.25C8.88805 3.25 8.0614 3.59241 7.4519 4.2019C6.84241 4.8114 6.5 5.63805 6.5 6.5V32.5C6.5 33.362 6.84241 34.1886 7.4519 34.7981C8.0614 35.4076 8.88805 35.75 9.75 35.75H29.25C30.112 35.75 30.9386 35.4076 31.5481 34.7981C32.1576 34.1886 32.5 33.362 32.5 32.5V13L22.75 3.25M21.125 5.6875L30.0625 14.625H21.125M14.5113 19.8575H26V31.3463L22.555 27.9013L17.9563 32.5L13.3575 27.9013L17.9563 23.3188" 
                                  fill="#1DBE5A"/>
                              </g>
                          </g>
                      </g>
                  </svg>
              </div>
              `,
      title: title,
      text: text,
      showCancelButton: true,
      confirmButtonText: confirmButtonText,
      cancelButtonText: 'ย้อนกลับ',
      customClass: {
        title: '!swal2-title-mt-20px',
        actions: '!swal2-actions-mt-20px',
        icon: '!swal2-icon-no-border',
        confirmButton: '!swal2-button-bg-green',
        cancelButton: '!swal2-button-bg-gray',
      },
    }).then((result) => {
      if (result.isConfirmed && this.appraisalPms) {
        this.saveApi(approveStatus)
      } else if (result.dismiss === Swal.DismissReason.cancel) {
        Swal.fire({
          title: 'ยกเลิก!',
          text: 'การบันทึกถูกยกเลิก',
          icon: 'error',
          customClass: {
            confirmButton: '!swal2-button-bg-danger',
          }
        })
      }
    })
  }
  saveApi(approveStatus: string) {
    let body = this.appraisalPms.data
    switch (this.appraisalPms.data?.currentStep) {
      case ("0"): {
        body = {
          ...this.appraisalPms.data,
          apsassessyStatus: approveStatus
        }
        break
      }
      case ("1"): {
        body = {
          ...this.appraisalPms.data,
          apsapprove1Status: approveStatus
        }
        break
      }
      case ("2"): {
        body = {
          ...this.appraisalPms.data,
          apsapprove1Status: approveStatus == '0' ? approveStatus : this.appraisalPms.data.apsapprove1Status,
          apsapprove2Status: approveStatus
        }
        break
      }
      case ("3"): {
        body = {
          ...this.appraisalPms.data,

          apsapprove2Status: approveStatus == '0' ? approveStatus : this.appraisalPms.data.apsapprove2Status,
          apsapprove3Status: approveStatus
        }
        break
      }
      case ("4"): {
        body = {
          ...this.appraisalPms.data,
          apsapprove3Status: approveStatus == '0' ? approveStatus : this.appraisalPms.data.apsapprove3Status,
          apsapprove4Status: approveStatus
        }
        break
      }
      case ("5"): {
        body = {
          ...this.appraisalPms.data,
          apsapprove4Status: approveStatus == '0' ? approveStatus : this.appraisalPms.data.apsapprove4Status,
          apsapprove5Status: approveStatus
        }
        break
      }
      default: { return }
    }
    this.appraisalService.postPms(body).subscribe({
      next: response => {
        if (response.success) {
          Swal.fire({
            title: 'บันทึกสำเร็จ!',
            text: 'การประเมินของคุณถูกบันทึกแล้ว',
            icon: 'success',
            customClass: {
              confirmButton: '!swal2-button-bg-green',
            }
          });
          this.getAppraisalPmsForm()
        } else {
          Swal.fire({
            title: 'ยกเลิก!',
            text: 'การบันทึกถูกยกเลิก',
            icon: 'error',
            customClass: {
              confirmButton: '!swal2-button-bg-danger',
            }
          });
        }
      }, error: error => {
        Swal.fire({
          title: 'ยกเลิก!',
          text: 'การบันทึกถูกยกเลิก',
          icon: 'error',
          customClass: {
            confirmButton: '!swal2-button-bg-danger',
          }
        });
      }
    })
  }

  toggleAllParts(check: boolean) {
    this.part1show = check
    this.part2show = check
    this.part3show = check
    this.part4show = check
    this.part5show = check
    this.part6show = check
    this.part7show = check
    this.part8show = check
    this.part9show = check
    this.part10show = check
  }
}