import { ChangeDetectorRef, Component, EventEmitter, Input, Output } from '@angular/core';
import { ToastrService } from 'ngx-toastr';
import { PLModel, MyPLModel } from 'src/app/shared/model/pl.model';
import { FileService } from 'src/app/shared/services/file.service';
import { PLService } from 'src/app/shared/services/pl.service';

@Component({
  selector: 'app-evaluation-grouping',
  templateUrl: './evaluation-grouping.component.html',
  styleUrls: ['./evaluation-grouping.component.scss']
})
export class EvaluationGroupingComponent {
  @Output() sendPathTitle: EventEmitter<string[]> = new EventEmitter<string[]>();
  onEdit = false

  currentPage = 1
  page = Array.from({ length: 1 }, (_, i) => i + 1);
  pageSize = 10
  search = ""
  currentModal: 'add' | 'edit' | 'delete' | 'deleteGroup' = "add"

  selectedFile: File | null = null;
  selectedFileName: string = 'กรุณาเลือกไฟล์';

  numDataListChecked = 0
  isDataListChecked = false
  isDataListCheckedAll = false

  pl: { loading: boolean, select: PLModel, dataList: { check: boolean, data: PLModel }[] } = { loading: false, select: new MyPLModel(), dataList: [] }
  constructor(private toastr: ToastrService, private plService: PLService, private cdr: ChangeDetectorRef, private fileService: FileService) { }
  ngOnInit(): void {
    this.onSendPathTitle()
    this.getPLList()
  }
  onFileSelected(event: any) {
    this.selectedFile = event.target.files.length > 0 ? event.target.files[0] : null;
    this.selectedFileName = this.selectedFile?.name || "กรุณาเลือกไฟล์"
  }

  uploadFile() {
    if (!this.selectedFile) {
      alert('กรุณาเลือกไฟล์ก่อนอัปโหลด')
      return
    }
    const formData = new FormData();
    formData.append('file', this.selectedFile);
    this.pl.loading = true
    this.fileService.uploadExcel(formData, 'PMS_GROUP_ASSESSMENT').subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getPLList()
        } else {
          this.showAlert(response.message, 'error')
          this.pl.loading = false
          this.cdr.detectChanges()
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.pl.loading = false
        this.cdr.detectChanges()
      }
    })
  }
  downloadFile() {
    const fileName = 'IMPORT_PMS_GROUP_ASSESSMENT.xlsx'
    this.fileService.downloadTemplate(fileName).subscribe({
      next: response => {
        const url = window.URL.createObjectURL(response);
        const a = document.createElement("a");
        a.href = url;
        a.download = fileName;
        document.body.appendChild(a);
        a.click();
        document.body.removeChild(a);
        window.URL.revokeObjectURL(url);
      }, error: error => {
        this.showAlert(error.message, 'error')
      }
    })

  }
  getPLList() {
    this.pl.loading = true
    this.plService.getList().subscribe({
      next: response => {
        this.pl.dataList = response.map(x => ({ check: false, data: new MyPLModel(x) }))
        this.pl.loading = false
        this.isDataListCheckedAll = false
        this.dataListCheckAll()
        this.searchChange();
        this.cdr.detectChanges();
      }, error: error => {
        this.pl.loading = false
        console.error('Error fetching employee types:', error);
        this.cdr.detectChanges()
      }
    });
  }
  plListFilter() {
    return this.pl.dataList.filter(x => {
      const data = x.data
      const match = data.plId.toLowerCase().includes(this.search.toLowerCase()) || data.tdesc.toLowerCase().includes(this.search.toLowerCase())
      return match;
    });
  }
  selectPl(data: PLModel) {
    this.pl.select = new MyPLModel(data)
  }
  searchChange() {
    this.currentPage = 1
    this.page = Array.from({ length: Math.ceil(this.plListFilter().length / 10) }, (_, i) => i + 1)
    this.dataListCheck();
    this.cdr.detectChanges()
  }
  dataListCheckAll() {
    const selectAll = this.isDataListCheckedAll;
    this.plListFilter().forEach(x => x.check = selectAll);
    this.dataListCheck();
  }
  dataListCheck() {
    const dataCheck = this.plListFilter();
    this.isDataListCheckedAll = dataCheck.length ? dataCheck.every(x => x.check) : false;
    this.numDataListChecked = this.pl.dataList.filter(x => x.check).length
    this.isDataListChecked = Boolean(this.numDataListChecked)
  }
  onSendPathTitle() {
    if (this.onEdit) {
      this.sendPathTitle.emit(['การประเมินจัดการประสิทธิภาพ', 'การจัดการประเมิน', 'จัดกลุ่มประเมิน', 'แก้ไขกลุ่มการประเมิน']);
    } else if (!this.onEdit) {
      this.sendPathTitle.emit(['การประเมินจัดการประสิทธิภาพ', 'การจัดการประเมิน', 'จัดกลุ่มประเมิน']);
    }
  }

  showAlert(text: string, type: 'success' | 'error') {
    this.toastr[type](text, 'แจ้งเตือน', {
      timeOut: 3000,
      positionClass: 'toast-top-right',
    })
  }

}

