import { Injectable, OnDestroy } from '@angular/core';
import { Subject, BehaviorSubject, fromEvent } from 'rxjs';
import { takeUntil, debounceTime } from 'rxjs/operators';
import { Router } from '@angular/router';
import { MenuBodyModel, MyMenuBodyModel } from '../model/config-permission.model';
// Menu
export interface Menu {
  headTitle?: string;
  headTitle2?: string;
  path?: string;
  title?: string;
  icon?: string;
  type?: string;
  badgeValue?: string;
  badgeClass?: string;
  active?: boolean;
  selected?: boolean;
  bookmark?: boolean;
  children?: Menu[];
  children2?: Menu[];
  Menusub?: boolean;
  target?: boolean;
  show?: boolean
  id?: string
}

@Injectable({
  providedIn: 'root',
})
export class NavService implements OnDestroy {
  private unsubscriber: Subject<any> = new Subject();
  public screenWidth: BehaviorSubject<number> = new BehaviorSubject(
    window.innerWidth
  );

  // Search Box
  public search = false;

  // Language
  public language = false;

  // Mega Menu
  public megaMenu = false;
  public levelMenu = false;
  public megaMenuColapse: boolean = window.innerWidth < 1199 ? true : false;

  // Collapse Sidebar
  public collapseSidebar: boolean = window.innerWidth < 991 ? true : false;

  // For Horizontal Layout Mobile
  public horizontal: boolean = window.innerWidth < 991 ? false : true;

  // Full screen
  public fullScreen = false;
  active: any;

  constructor(private router: Router) {
    this.setScreenWidth(window.innerWidth);
    fromEvent(window, 'resize')
      .pipe(debounceTime(1000), takeUntil(this.unsubscriber))
      .subscribe((evt: any) => {
        this.setScreenWidth(evt.target.innerWidth);
        if (evt.target.innerWidth < 991) {
          this.collapseSidebar = true;
          this.megaMenu = false;
          this.levelMenu = false;
        }
        if (evt.target.innerWidth < 1199) {
          this.megaMenuColapse = true;
        }
      });
    if (window.innerWidth < 991) {
      // Detect Route change sidebar close
      this.router.events.subscribe((event) => {
        this.collapseSidebar = true;
        this.megaMenu = false;
        this.levelMenu = false;
      });
    }
  }

  ngOnDestroy() {
    this.unsubscriber.next;
    this.unsubscriber.complete();
  }

  private setScreenWidth(width: number): void {
    this.screenWidth.next(width);
  }

  MENUITEMS: Menu[] = [
    {
      title: 'การประเมินผล',
      type: 'sub',
      selected: false,
      active: false,
      path: '',
      id: 'm1',
      show: false,
      children: [
        // { id: 'm10', path: '/pms-evaluation', title: 'ประเมินตนเอง', type: 'link', show: false },
        { id: 'm11', path: '/self-evaluation', title: 'ประเมินตนเอง', type: 'link', show: false },
        { id: 'm12', path: '/supervisor-evaluation', title: 'ประเมินโดยหัวหน้า', type: 'link', show: false },
      ],
    },
    {
      title: 'การจัดการข้อมูลองค์กร',
      type: 'sub',
      selected: false,
      active: false,
      path: '',
      id: 'm2',
      show: false,
      children: [
        { id: 'm21', path: '/company-registration', title: 'ทะเบียนบริษัท', type: 'link', show: false },
        { id: 'm22', path: '/job-description', title: 'ข้อมูลลักษณะงาน', type: 'link', show: false },
        { id: 'm23', path: '/employee-registration', title: 'ทะเบียนพนักงาน', type: 'link', show: false },
        { id: 'm24', path: '/day-type-registry', title: 'ทะเบียนประเภทวัน', type: 'link', show: false },
        { id: 'm25', path: '/account-settings', title: 'ตั้งค่าชื่อผู้ใช้', type: 'link', show: false },
        { id: 'm26', path: '/role-permission-config', title: 'กำหนดสิทธิการเข้าใช้งาน', type: 'link', show: false },
      ],
    },
    {
      title: 'การจัดการรายละเอียดงาน',
      type: 'sub',
      selected: false,
      active: false,
      path: '',
      id: 'm3',
      show: false,
      children: [
        { id: 'm31', path: '/job-detail-management', title: 'ข้อมูลทั่วไป', type: 'link', show: false },
        { id: 'm32', path: '/command-structure', title: 'โครงสร้างสายการบังคับบัญชา', type: 'link', show: false },
        { id: 'm33', path: '/job-detail', title: 'รายละเอียดของงาน', type: 'link', show: false },
        { id: 'm34', path: '/job-qualifications', title: 'คุณสมบัติที่จำเป็น', type: 'link', show: false },
        { id: 'm35', path: '/job-competency', title: 'ความสามารถในตำเเหน่งงาน', type: 'link', show: false },
        { id: 'm36', path: '/job-position-indicators', title: 'ตัวชี้วัดของตำแหน่งงาน', type: 'link', show: false },
      ],
    },

    {
      title: 'การประเมินสมรรถนะ',
      type: 'sub',
      selected: false,
      active: false,
      path: '',
      id: 'm4',
      show: false,
      children: [
        { id: 'm41', path: '/name-registration', title: 'ทะเบียนกำหนดชื่อ', type: 'link', show: false },
        { id: 'm42', path: '/grade-registration', title: 'ทะเบียนเกรด', type: 'link', show: false },
        { id: 'm43', path: '/tool-register', title: 'ทะเบียนเครื่องมือ', type: 'link', show: false },
        { id: 'm44', path: '/course-registration', title: 'ทะเบียนหลักสูตร', type: 'link', show: false },
        { id: 'm45', path: '/idp-development-plan', title: 'แผนพัฒนา IDP', type: 'link', show: false },
        { id: 'm46', path: '/competency-management', title: 'การจัดการสมรรถนะ', type: 'link', show: false },
        { id: 'm47', path: '/evaluation-cycle-manager', title: 'การจัดการรอบการประเมิน', type: 'link', show: false },
        { id: 'm48', path: '/setting-competency', title: 'การตั้งค่า', type: 'link', show: false }
      ],
    },
    {
      title: 'การประเมินจัดการประสิทธิภาพ',
      type: 'sub',
      selected: false,
      active: false,
      path: '',
      id: 'm5',
      show: false,
      children: [
        { id: 'm51', path: '/name-registration-perfomance', title: 'ทะเบียนกำหนดชื่อ', type: 'link', show: false },
        { id: 'm52', path: '/grade-registration-sub', title: 'ทะเบียนเกรด', type: 'link', show: false },
        { id: 'm53', path: '/time-attendance', title: 'ทะเบียนการประเมินเวลาทำงาน', type: 'link', show: false },
        { id: 'm54', path: '/evaluation-factors', title: 'ปัจจัยการประเมินผล', type: 'link', show: false },
        { id: 'm55', path: '/assessment-management', title: 'การจัดการการประเมิน', type: 'link', show: false },
        { id: 'm56', path: '/evaluation-cycle-performance', title: 'รอบการประเมิน', type: 'link', show: false },
        { id: 'm57', path: '/setting-performance-evalution', title: 'การตั้งค่า', type: 'link', show: false }
      ],
    },
  ];
  // Array
  items = new BehaviorSubject<Menu[]>(this.MENUITEMS);
}
